@@ -6,7 +6,7 @@
   .gibbs_IMFA        <- function(Q, data, iters, N, P, G, mu.zero, rho, sigma.l, learn.alpha, discount, mu, tune.zeta,
                                  a.hyper, sigma.mu, burnin, thinning, d.hyper, learn.d, uni.type, uni.prior, trunc.G,
                                  ind.slice, psi.alpha, psi.beta, verbose, sw, cluster, IM.lab.sw, zeta, kappa, ...) {
-                                #ind.slice, psi.alpha, psi.beta, verbose, sw, cluster, IM.lab.sw, zeta, kappa, thresh, ...) {
+                                #ind.slice, psi.alpha, psi.beta, verbose, sw, cluster, IM.lab.sw, zeta, kappa, thresh, exchange, ...) {
 
   # Define & initialise variables
     start.time       <- proc.time()
@@ -85,6 +85,7 @@
     nn               <- tabulate(z, nbins=trunc.G)
     nn0              <- nn > 0
     nn.ind           <- which(nn > 0)
+    G.non            <- length(nn.ind)
     one.uni          <- is.element(uni.type, c("constrained", "single"))
     .sim_psi_inv     <- switch(EXPR=uni.type,  unconstrained=.sim_psi_uu,   isotropic=.sim_psi_uc,
                                                constrained=.sim_psi_cu,     single=.sim_psi_cc)
@@ -122,7 +123,6 @@
       slinf          <- rep(-Inf, N)
      #if(thresh)  {
      #  ksi2         <- ksi
-     #  log.ksi2     <- log.ksi
      #}
     } else slinf     <- c(-Inf,  0L)
    #if((noLearn      <-
@@ -139,13 +139,20 @@
       storage        <- is.element(iter, iters)
 
     # Mixing Proportions & Re-ordering
+     #if(!exchange)   {
       Vs             <- .sim_vs_inf(alpha=pi.alpha, nn=nn[Gs], N=N, discount=discount, len=G, lseq=Gs)
       pi.prop        <- .sim_pi_inf(Vs, len=G)
+      prev.prod      <- 1 - sum(pi.prop)
+     #} else          {
+     #  piX          <- .sim_pi_infX(nn=nn[nn0], Kn=G.non, G=G, alpha=pi.alpha, discount=discount)
+     #  pi.prop      <- piX$pi.prop
+     #  prev.prod    <- piX$prev.prod
+     #}
       index          <- order(pi.prop, decreasing=TRUE)
-      prev.prod      <- pi.prop[G]  * (1/Vs[G] - 1)
       GI             <- which(Gs[index] == G)
       pi.prop        <- pi.prop[index]
       Vs             <- Vs[index]
+     #Vs             <- if(!exchange) Vs[index] else rep(0L, G)
       mu[,Gs]        <- mu[,index, drop=FALSE]
       lmat[,,Gs]     <- lmat[,,index, drop=FALSE]
       psi.inv[,Gs]   <- psi.inv[,index, drop=FALSE]
@@ -188,8 +195,8 @@
      #  TRX          <- ifelse(noLearn, TRX, .slice_threshold(N, pi.alpha, discount, MPFR=pi.alpha == 0))
      #}
       if(!ind.slice)  {
-       #ksi          <- if(thresh) pmin(pi.prop, TRX) else pi.prop
         ksi          <- pi.prop
+       #ksi          <- if(thresh) pmin(pi.prop, TRX) else pi.prop
         log.ksi      <- log(ksi)
       }
      #} else if(thresh)   {
@@ -202,12 +209,13 @@
       if(ind.slice)   {
         G.new        <- sum(min.u    < ksi)
         G.trunc      <- G.new < G.old
+       #while(G  < G.new && (ifelse(exchange, prev.prod >= min.u, Vs[GI] != 1) || pi.prop[GI] != 0)) {
         while(G  < G.new && (Vs[GI] != 1 || pi.prop[GI] != 0)) {
           newVs      <- .sim_vs_inf(alpha=pi.alpha, discount=discount, len=1L, lseq=G + 1L)
           Vs         <- c(Vs,      newVs)
-          pi.prop    <- c(pi.prop, newVs * prev.prod)
+          pi.prop    <- c(pi.prop, newVs  * prev.prod)
           GI    <- G <- G + 1L
-          prev.prod  <- pi.prop[G]  * (1/Vs[G] - 1)
+          prev.prod  <- 1 - sum(pi.prop)
         }
         G            <- ifelse(G.trunc, G.new, G)
         Gs           <- seq_len(G)
@@ -219,16 +227,17 @@
         cum.pi       <- sum(pi.prop)
         u.max        <- 1 - min.u
         G.trunc      <- cum.pi > u.max
-        while(cum.pi <= u.max && trunc.G > G && (pi.prop[GI] != 0 || Vs[GI] != 1)) {
+       #while(cum.pi  < u.max && trunc.G > G && (pi.prop[GI] != 0 || ifelse(exchange, prev.prod >= min.u, Vs[GI] != 1))) {
+        while(cum.pi  < u.max && trunc.G > G && (pi.prop[GI] != 0 || Vs[GI] != 1)) {
           newVs      <- .sim_vs_inf(alpha=pi.alpha, discount=discount, len=1L, lseq=G + 1L)
           Vs         <- c(Vs,      newVs)
           newPis     <- newVs  *   prev.prod
-          pi.prop    <- c(pi.prop, newPis)
-          cum.pi     <- cum.pi +   newPis
-          ksi        <- pi.prop
+          pi.prop    <-
+          ksi        <- c(pi.prop, newPis)
           log.ksi    <- c(log.ksi, log(newPis))
+          cum.pi     <- cum.pi +   newPis
           GI    <- G <- G + 1L
-          prev.prod  <- pi.prop[G] * (1/Vs[G] - 1)
+          prev.prod  <- 1 - cum.pi
         }
         G            <- ifelse(G.trunc, which.max(cumsum(pi.prop) > u.max), G)
         Gs           <- seq_len(G)
