@@ -19,7 +19,7 @@
 #' Must be given as a list of length \code{G} of vectors of length corresponding to \code{Q} when \code{G>1}. Can be given either as such a list or simply a vector of length \code{Q} when \code{G=1}. Alternatively, a single integer can be supplied, common across all loadings columns across all clusters. In any case, \code{non.zero} will be affected by \code{forceQg=TRUE} by default (see below).
 #' @param forceQg A logical indicating whether the upper limit on the number of cluster-specific factors \code{Q} is enforced. Defaults to \code{TRUE} for \code{sim_IMIFA_data}, but is always \code{FALSE} for \code{sim_IMIFA_model}. Note that when \code{forceQg=TRUE} is invoked, \code{non.zero} (see above) is also affected.
 #' @param method A switch indicating whether the mixture to be simulated from is the conditional distribution of the data given the latent variables (default), or simply the marginal distribution of the data.
-#' @param res An object of class "\code{Results_IMIFA}" generated by \code{\link{get_IMIFA_results}}.
+#' @param res An object of class \code{"Results_IMIFA"} generated by \code{\link{get_IMIFA_results}}.
 #'
 #' @return Invisibly returns a \code{data.frame} with \code{N} observations (rows) of \code{P} variables (columns). The true values of the parameters which generated these data are also stored as attributes.
 #' @details \code{sim_IMIFA_model} is a simple wrapper to \code{sim_IMIFA_data} which uses the estimated parameters of a fitted IMIFA related model, as generated by \code{\link{get_IMIFA_results}}. The necessary parameters must have been originally stored via \code{\link{storeControl}} in the creation of \code{res}.
@@ -27,13 +27,13 @@
 #'
 #' Missing values are not allowed in any of \code{pis}, \code{mu}, \code{psi}, \code{loadings}, \code{scores} & \code{nn}.
 #' @export
-#' @references Murphy, K., Viroli, C., and Gormley, I. C. (2019) Infinite mixtures of infinite factor analysers, \emph{Bayesian Analysis}, 1-27. <\href{https://projecteuclid.org/euclid.ba/1570586978}{doi:10.1214/19-BA1179}>.
+#' @references Murphy, K., Viroli, C., and Gormley, I. C. (2020) Infinite mixtures of infinite factor analysers, \emph{Bayesian Analysis}, 15(3): 937-963. <\href{https://projecteuclid.org/euclid.ba/1570586978}{doi:10.1214/19-BA1179}>.
 #' @keywords utility
 #' @importFrom Rfast "is.symmetric"
 #' @name sim_IMIFA
 #' @rdname sim_IMIFA
 #'
-#' @author Keefe Murphy - <\email{keefe.murphy@@ucd.ie}>
+#' @author Keefe Murphy - <\email{keefe.murphy@@mu.ie}>
 #' @usage
 #' sim_IMIFA_data(N = 300L,
 #'                G = 3L,
@@ -53,10 +53,10 @@
 #' # Simulate 100 observations from 3 balanced clusters with cluster-specific numbers of latent factors
 #' # Specify isotropic uniquenesses within each cluster
 #' # Supply cluster means directly
-#' sim_data <- sim_IMIFA_data(N=100, G=3, P=20, Q=c(2, 2, 5), psi=1:3,
-#'                            mu=matrix(rnorm(60, -2 + 1:3, 1), nrow=20, ncol=3, byrow=TRUE))
+#' sim_data  <- sim_IMIFA_data(N=100, G=3, P=20, Q=c(2, 2, 5), psi=1:3,
+#'                             mu=matrix(rnorm(60, -2 + 1:3, 1), nrow=20, ncol=3, byrow=TRUE))
 #' names(attributes(sim_data))
-#' labels   <- attr(sim_data, "Labels")
+#' labels    <- attr(sim_data, "Labels")
 #'
 #' # Visualise the data in two-dimensions
 #' plot(cmdscale(dist(sim_data), k=2), col=labels)
@@ -65,7 +65,11 @@
 #' pairs(sim_data[,sample(1:20, 5)], col=labels)
 #'
 #' \donttest{# Fit a MIFA model to this data
-#' # tmp    <- mcmc_IMIFA(sim_data, method="MIFA", range.G=3, n.iters=5000)}
+#' # tmp     <- mcmc_IMIFA(sim_data, method="MIFA", range.G=3, n.iters=5000)
+#'
+#' # Simulate from this model
+#' # res     <- get_IMIFA_results(tmp, zlabels=labels)
+#' # sim_mod <- sim_IMIFA_model(res)}
 #' @seealso \code{\link{mcmc_IMIFA}} for fitting an IMIFA related model to the simulated data set.
 #'
 #' \code{\link{get_IMIFA_results}} for generating input for \code{sim_IMIFA_model}.
@@ -140,7 +144,7 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
     if(anyNA(scores))                     stop("Missing values are not allowed in 'scores'", call.=FALSE)
   }
   Q.warn       <- pmin(pmax(0L, nn - 1L), Ledermann(P, isotropic=!psimiss && length(unique(psisup[,1L])) == 1))
-  if(any(Q.ind <- Q > Q.warn))        {
+  if(any(Q.ind <- (Q > Q.warn)))      {
     if(length(forceQg)  != 1 ||
        !is.logical(forceQg))              stop("'forceQg' must be a single logical indicator", call.=FALSE)
     warn.Q     <- paste0("Too many factors generated relative to P=", P, " and the number of observations in cluster", ifelse(sum(Q.ind) > 1, "s ", " "), paste(which(Q.ind), paste0("(", nn[Q.ind], ")"), collapse=", "))
